/// <reference types="express" />
/// <reference types="node" />
/// <reference types="ws" />
import * as express from 'express';
import { Server as HttpServer } from 'http';
import * as WebSocket from 'ws';
import { Handle } from '@dojo/interfaces/core';
import Node from './executors/Node';
import { Message } from './channels/Base';
export interface InternObject {
    readonly stopped: boolean;
    readonly basePath: string;
    readonly executor: Node;
    handleMessage(message: Message): Promise<any>;
}
export interface InternRequest extends express.Request {
    readonly intern: InternObject;
}
export interface InternResponse extends express.Response {
    readonly intern: InternObject;
}
export interface InternRequestHandler {
    (req: InternRequest, res: InternResponse, next: express.NextFunction): any;
}
export interface InternErrorRequestHandler {
    (err: any, req: InternRequest, res: InternResponse, next: express.NextFunction): any;
}
export default class Server implements ServerProperties {
    readonly executor: Node;
    basePath: string;
    port: number;
    runInSync: boolean;
    socketPort: number;
    readonly stopped: boolean;
    protected _app: express.Express | null;
    protected _httpServer: HttpServer | null;
    protected _sessions: {
        [id: string]: {
            listeners: ServerListener[];
        };
    };
    protected _wsServer: WebSocket.Server | null;
    constructor(options: ServerOptions);
    start(): Promise<void>;
    stop(): Promise<any[]>;
    subscribe(sessionId: string, listener: ServerListener): Handle;
    private _getSession(sessionId);
    private _handleMessage(message);
    private _handleWebSocket(client);
    private _publish(message);
}
export interface ServerProperties {
    basePath: string;
    executor: Node;
    port: number;
    runInSync: boolean;
    socketPort: number;
}
export interface ServerListener {
    (name: string, data?: any): void;
}
export declare type ServerOptions = Partial<ServerProperties> & {
    executor: Node;
};
