(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "@dojo/core/lang"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var lang_1 = require("@dojo/core/lang");
    var configPathSeparator = '@';
    function prefix(message, prefix) {
        return message
            .split('\n')
            .map(function (line) { return prefix + line; })
            .join('\n');
    }
    exports.prefix = prefix;
    function loadConfig(configPath, loadText, args, childConfig) {
        return _loadConfig(configPath, loadText, args, childConfig).then(function (config) {
            delete config.config;
            delete config.extends;
            if (!(args && (args.showConfigs || args.help))) {
                delete config.configs;
            }
            return config;
        });
    }
    exports.loadConfig = loadConfig;
    function _loadConfig(configPath, loadText, args, childConfig) {
        return loadText(configPath)
            .then(function (text) {
            var config = parseJson(text);
            if (config.extends) {
                var parts = configPath.split('/');
                var _a = splitConfigPath(config.extends), configFile = _a.configFile, childConfig_1 = _a.childConfig;
                var extensionPath = parts
                    .slice(0, parts.length - 1)
                    .concat(configFile)
                    .join('/');
                return _loadConfig(extensionPath, loadText, undefined, childConfig_1).then(function (extension) {
                    Object.keys(config)
                        .filter(function (key) { return key !== 'configs'; })
                        .forEach(function (key) {
                        extension[key] = config[key];
                    });
                    if (config.configs) {
                        if (extension.configs == null) {
                            extension.configs = {};
                        }
                        Object.keys(config.configs).forEach(function (key) {
                            extension.configs[key] = config.configs[key];
                        });
                    }
                    return extension;
                });
            }
            return config;
        })
            .then(function (config) {
            if (args && (args.showConfigs || args.help)) {
                return config;
            }
            if (childConfig) {
                var mixinConfig_1 = function (childConfig) {
                    var configs = Array.isArray(childConfig)
                        ? childConfig
                        : [childConfig];
                    configs.forEach(function (childConfig) {
                        var child = config.configs[childConfig];
                        if (!child) {
                            throw new Error("Unknown child config \"" + childConfig + "\"");
                        }
                        if (child.extends) {
                            mixinConfig_1(child.extends);
                        }
                        Object.keys(child)
                            .filter(function (key) { return key !== 'node' && key !== 'browser'; })
                            .forEach(function (key) {
                            config[key] = child[key];
                        });
                        ['node', 'browser'].forEach(function (key) {
                            if (child[key]) {
                                if (config[key]) {
                                    lang_1.mixin(config[key], child[key]);
                                }
                                else {
                                    config[key] = child[key];
                                }
                            }
                        });
                    });
                };
                mixinConfig_1(childConfig);
            }
            return config;
        })
            .then(function (config) {
            if (args) {
                var resources = [
                    'plugins',
                    'reporters',
                    'suites'
                ];
                resources
                    .filter(function (resource) { return resource in args; })
                    .forEach(function (resource) {
                    var environments = [
                        'node',
                        'browser'
                    ];
                    environments
                        .filter(function (environment) { return config[environment]; })
                        .forEach(function (environment) {
                        delete config[environment][resource];
                    });
                });
                lang_1.deepMixin(config, args);
            }
            return config;
        });
    }
    function getConfigDescription(config, prefix) {
        if (prefix === void 0) { prefix = ''; }
        var description = '';
        if (config.description) {
            description += "" + prefix + config.description + "\n\n";
        }
        if (config.configs) {
            description += prefix + "Configs:\n";
            var width_1 = Object.keys(config.configs).reduce(function (width, name) {
                return Math.max(width, name.length);
            }, 0);
            var lines = Object.keys(config.configs).map(function (name) {
                var child = config.configs[name];
                while (name.length < width_1) {
                    name += ' ';
                }
                var line = "  " + name;
                if (child.description) {
                    line += " (" + child.description + ")";
                }
                return "" + prefix + line;
            });
            description += lines.join('\n');
        }
        return description;
    }
    exports.getConfigDescription = getConfigDescription;
    function normalizePathEnding(path) {
        if (path && path.length > 0 && path[path.length - 1] !== '/') {
            return path + "/";
        }
        return path;
    }
    exports.normalizePathEnding = normalizePathEnding;
    function parseArgs(rawArgs) {
        var parsedArgs = {};
        for (var _i = 0, rawArgs_1 = rawArgs; _i < rawArgs_1.length; _i++) {
            var arg = rawArgs_1[_i];
            var name_1 = arg;
            var value = void 0;
            var args = parsedArgs;
            var eq = arg.indexOf('=');
            if (eq !== -1) {
                name_1 = arg.slice(0, eq);
                value = arg.slice(eq + 1);
            }
            if (name_1.indexOf('.') !== -1) {
                var parts = name_1.split('.');
                var head = parts.slice(0, parts.length - 1);
                name_1 = parts[parts.length - 1];
                for (var _a = 0, head_1 = head; _a < head_1.length; _a++) {
                    var part = head_1[_a];
                    if (!args[part]) {
                        args[part] = {};
                    }
                    args = args[part];
                }
            }
            if (typeof value === 'undefined') {
                args[name_1] = true;
            }
            else {
                if (!(name_1 in args)) {
                    args[name_1] = value;
                }
                else if (!Array.isArray(args[name_1])) {
                    args[name_1] = [args[name_1], value];
                }
                else {
                    args[name_1].push(value);
                }
            }
        }
        return parsedArgs;
    }
    exports.parseArgs = parseArgs;
    function parseJson(json) {
        return JSON.parse(removeComments(json));
    }
    exports.parseJson = parseJson;
    function parseValue(name, value, parser, requiredProperty) {
        switch (parser) {
            case 'boolean':
                if (typeof value === 'boolean') {
                    return value;
                }
                if (value === 'true') {
                    return true;
                }
                if (value === 'false') {
                    return false;
                }
                throw new Error("Non-boolean value \"" + value + "\" for " + name);
            case 'number':
                var numValue = Number(value);
                if (!isNaN(numValue)) {
                    return numValue;
                }
                throw new Error("Non-numeric value \"" + value + "\" for " + name);
            case 'regexp':
                if (typeof value === 'string') {
                    return new RegExp(value);
                }
                if (value instanceof RegExp) {
                    return value;
                }
                throw new Error("Non-regexp value \"" + value + "\" for " + name);
            case 'object':
                if (typeof value === 'string') {
                    try {
                        value = value ? JSON.parse(value) : {};
                    }
                    catch (error) {
                        if (!requiredProperty) {
                            throw new Error("Non-object value \"" + value + "\" for " + name);
                        }
                        value = (_a = {}, _a[requiredProperty] = value, _a);
                    }
                }
                if (Object.prototype.toString.call(value) === '[object Object]') {
                    if (requiredProperty && !value[requiredProperty]) {
                        throw new Error("Invalid value \"" + JSON.stringify(value) + "\" for " + name + ": missing '" + requiredProperty + "' property");
                    }
                    return value;
                }
                throw new Error("Non-object value \"" + value + "\" for " + name);
            case 'object[]':
                if (!value) {
                    value = [];
                }
                if (!Array.isArray(value)) {
                    value = [value];
                }
                return value.map(function (item) {
                    return parseValue(name, item, 'object', requiredProperty);
                });
            case 'string':
                if (typeof value === 'string') {
                    return value;
                }
                throw new Error("Non-string value \"" + value + "\" for " + name);
            case 'string[]':
                if (!value) {
                    value = [];
                }
                if (typeof value === 'string') {
                    value = [value];
                }
                if (Array.isArray(value) &&
                    value.every(function (v) { return typeof v === 'string'; })) {
                    return value;
                }
                throw new Error("Non-string[] value \"" + value + "\" for " + name);
            default:
                if (typeof parser === 'function') {
                    return parser(value);
                }
                else {
                    throw new Error('Parser must be a valid type name');
                }
        }
        var _a;
    }
    exports.parseValue = parseValue;
    function pullFromArray(haystack, needle) {
        var removed = [];
        var i = 0;
        while ((i = haystack.indexOf(needle, i)) > -1) {
            removed.push(haystack.splice(i, 1)[0]);
        }
        return removed;
    }
    exports.pullFromArray = pullFromArray;
    function isTask(value) {
        return value && 'cancel' in value && typeof value.cancel === 'function';
    }
    exports.isTask = isTask;
    function splitConfigPath(path, separator) {
        if (separator === void 0) { separator = '/'; }
        var lastSep = path.lastIndexOf(configPathSeparator);
        if (lastSep === 0) {
            return { configFile: '', childConfig: path.slice(1) };
        }
        if (lastSep === -1 || path[lastSep - 1] === separator) {
            return { configFile: path };
        }
        return {
            configFile: path.slice(0, lastSep),
            childConfig: path.slice(lastSep + 1)
        };
    }
    exports.splitConfigPath = splitConfigPath;
    function stringify(object, indent) {
        return JSON.stringify(object, serializeReplacer, indent);
    }
    exports.stringify = stringify;
    function serializeReplacer(_key, value) {
        if (!value) {
            return value;
        }
        if (value instanceof RegExp) {
            return value.source;
        }
        if (typeof value === 'function') {
            return value.toString();
        }
        return value;
    }
    function removeComments(text) {
        var state = 'default';
        var i = 0;
        var chars = text.split('');
        while (i < chars.length) {
            switch (state) {
                case 'block-comment':
                    if (chars[i] === '*' && chars[i + 1] === '/') {
                        chars[i] = ' ';
                        chars[i + 1] = ' ';
                        state = 'default';
                        i += 2;
                    }
                    else if (chars[i] !== '\n') {
                        chars[i] = ' ';
                        i += 1;
                    }
                    else {
                        i += 1;
                    }
                    break;
                case 'line-comment':
                    if (chars[i] === '\n') {
                        state = 'default';
                    }
                    else {
                        chars[i] = ' ';
                    }
                    i += 1;
                    break;
                case 'string':
                    if (chars[i] === '"') {
                        state = 'default';
                        i += 1;
                    }
                    else if (chars[i] === '\\' && chars[i + 1] === '\\') {
                        i += 2;
                    }
                    else if (chars[i] === '\\' && chars[i + 1] === '"') {
                        i += 2;
                    }
                    else {
                        i += 1;
                    }
                    break;
                default:
                    if (chars[i] === '"') {
                        state = 'string';
                        i += 1;
                    }
                    else if (chars[i] === '/' && chars[i + 1] === '*') {
                        chars[i] = ' ';
                        chars[i + 1] = ' ';
                        state = 'block-comment';
                        i += 2;
                    }
                    else if (chars[i] === '/' && chars[i + 1] === '/') {
                        chars[i] = ' ';
                        chars[i + 1] = ' ';
                        state = 'line-comment';
                        i += 2;
                    }
                    else {
                        i += 1;
                    }
            }
        }
        return chars.join('');
    }
});
//# sourceMappingURL=util.js.map