import { CoverageMap } from 'istanbul-lib-coverage';
import { Instrumenter } from 'istanbul-lib-instrument';
import { MapStore } from 'istanbul-lib-source-maps';
import Task from '@dojo/core/async/Task';
import Command from '@theintern/leadfoot/Command';
import Tunnel, { TunnelOptions, DownloadProgressEvent } from '@theintern/digdug/Tunnel';
import { BrowserStackOptions } from '@theintern/digdug/BrowserStackTunnel';
import { SeleniumOptions } from '@theintern/digdug/SeleniumTunnel';
import Executor, { Config as BaseConfig, Events as BaseEvents, Plugins } from './Executor';
import Environment from '../Environment';
import Server from '../Server';
import Suite from '../Suite';
import { RuntimeEnvironment } from '../types';
export default class Node extends Executor<Events, Config, NodePlugins> {
    server: Server;
    tunnel: Tunnel;
    protected _coverageMap: CoverageMap;
    protected _coverageFiles: string[];
    protected _loadingFunctionalSuites: boolean;
    protected _instrumentBasePath: string;
    protected _instrumenter: Instrumenter;
    protected _sourceMaps: MapStore;
    protected _instrumentedMaps: MapStore;
    protected _unhookRequire: null | (() => void);
    protected _sessionSuites: Suite[];
    constructor(options?: {
        [key in keyof Config]?: any;
    });
    readonly coverageMap: CoverageMap;
    readonly environment: RuntimeEnvironment;
    readonly instrumentedMapStore: MapStore;
    readonly sourceMapStore: MapStore;
    readonly hasCoveredFiles: boolean;
    readonly suites: Suite[];
    addSuite(factory: (parentSuite: Suite) => void): void;
    getTunnel(name: string): typeof Tunnel;
    instrumentCode(code: string, filename: string): string;
    loadScript(script: string | string[]): Task<void>;
    registerTunnel(name: string, Ctor: typeof Tunnel): void;
    shouldInstrumentFile(filename: string): boolean;
    protected _afterRun(): Task<void>;
    protected _beforeRun(): Task<boolean>;
    protected _createSessionSuites(): Task<void>;
    protected _loadFunctionalSuites(): Task<void>;
    protected _loadSuites(): Task<void>;
    protected _processOption(name: keyof Config, value: any, addToExisting: boolean): void;
    protected _resolveConfig(): Task<void>;
    protected _runTests(): Task<void>;
    protected _runRemoteTests(): Task<void>;
    protected _setInstrumentationHooks(): void;
    protected _removeInstrumentationHooks(): void;
}
export interface NodePlugins extends Plugins {
    tunnel: typeof Tunnel;
}
export interface Config extends BaseConfig {
    capabilities: {
        name?: string;
        build?: string;
        [key: string]: any;
    };
    connectTimeout: never;
    functionalTimeouts: {
        connectTimeout?: number;
        find?: number;
        executeAsync?: number;
        pageLoad?: number;
    };
    coverage: false | string[];
    environments: EnvironmentSpec[];
    excludeInstrumentation: never;
    functionalBaseUrl?: string;
    functionalCoverage: boolean;
    functionalSuites: string[];
    instrumenterOptions: {
        [key: string]: any;
    };
    leaveRemoteOpen: boolean | 'fail';
    maxConcurrency: number;
    proxy?: string;
    serveOnly: boolean;
    serverPort: number;
    serverUrl: string;
    runInSync: boolean;
    socketPort?: number;
    tunnel: string;
    tunnelOptions: TunnelOptions | BrowserStackOptions | SeleniumOptions;
}
export interface Remote extends Command<any> {
    environmentType?: Environment;
    setHeartbeatInterval(delay: number): Command<any>;
}
export interface EnvironmentSpec {
    browserName: string;
    [key: string]: any;
}
export interface TunnelMessage {
    tunnel: Tunnel;
    progress?: DownloadProgressEvent;
    status?: string;
}
export interface Events extends BaseEvents {
    serverEnd: Server;
    serverStart: Server;
    tunnelDownloadProgress: TunnelMessage;
    tunnelStart: TunnelMessage;
    tunnelStatus: TunnelMessage;
    tunnelStop: TunnelMessage;
}
