(function (factory) {
    if (typeof module === "object" && typeof module.exports === "object") {
        var v = factory(require, exports);
        if (v !== undefined) module.exports = v;
    }
    else if (typeof define === "function" && define.amd) {
        define(["require", "exports", "fs", "path", "glob", "shell-quote", "@dojo/core/async/Task", "@dojo/core/lang", "@dojo/shim/global", "../common/util"], factory);
    }
})(function (require, exports) {
    "use strict";
    Object.defineProperty(exports, "__esModule", { value: true });
    var fs_1 = require("fs");
    var path_1 = require("path");
    var glob_1 = require("glob");
    var shell_quote_1 = require("shell-quote");
    var Task_1 = require("@dojo/core/async/Task");
    var lang_1 = require("@dojo/core/lang");
    var global_1 = require("@dojo/shim/global");
    var util_1 = require("../common/util");
    var process = global_1.default.process;
    function expandFiles(patterns) {
        if (!patterns) {
            patterns = [];
        }
        else if (!Array.isArray(patterns)) {
            patterns = [patterns];
        }
        var excludes = [];
        var includes = [];
        var paths = [];
        for (var _i = 0, patterns_1 = patterns; _i < patterns_1.length; _i++) {
            var pattern = patterns_1[_i];
            if (pattern[0] === '!') {
                excludes.push(pattern.slice(1));
            }
            else {
                if (glob_1.hasMagic(pattern)) {
                    includes.push(pattern);
                }
                else {
                    paths.push(pattern);
                }
            }
        }
        return includes
            .map(function (pattern) { return glob_1.sync(pattern, { ignore: excludes }); })
            .reduce(function (allFiles, files) { return allFiles.concat(files); }, paths);
    }
    exports.expandFiles = expandFiles;
    function getConfig(file) {
        var args = {};
        if (process.env['INTERN_ARGS']) {
            lang_1.mixin(args, util_1.parseArgs(shell_quote_1.parse(process.env['INTERN_ARGS'])));
        }
        if (process.argv.length > 2) {
            lang_1.mixin(args, util_1.parseArgs(process.argv.slice(2)));
        }
        if (file) {
            args.config = file;
        }
        var load;
        if (args.config) {
            var _a = util_1.splitConfigPath(args.config, path_1.sep), configFile = _a.configFile, childConfig = _a.childConfig;
            file = configFile || 'intern.json';
            load = util_1.loadConfig(file, loadText, args, childConfig);
        }
        else {
            file = 'intern.json';
            load = util_1.loadConfig('intern.json', loadText, args).catch(function (error) {
                if (error.code === 'ENOENT') {
                    return args;
                }
                throw error;
            });
        }
        return load.then(function (config) { return ({ config: config, file: file }); });
    }
    exports.getConfig = getConfig;
    function normalizePath(path) {
        return path_1.normalize(path).replace(/\\/g, '/');
    }
    exports.normalizePath = normalizePath;
    function readSourceMap(sourceFile, code) {
        if (!code) {
            code = fs_1.readFileSync(sourceFile, { encoding: 'utf8' });
        }
        var match;
        var lastNewline = code.lastIndexOf('\n', code.length - 2);
        var lastLine = code.slice(lastNewline + 1);
        if ((match = sourceMapRegEx.exec(lastLine))) {
            if (match[1]) {
                return JSON.parse(new Buffer(match[2], 'base64').toString('utf8'));
            }
            else {
                var mapFile = path_1.join(path_1.dirname(sourceFile), match[2]);
                return JSON.parse(fs_1.readFileSync(mapFile, { encoding: 'utf8' }));
            }
        }
    }
    exports.readSourceMap = readSourceMap;
    function loadText(path) {
        return new Task_1.default(function (resolve, reject) {
            fs_1.readFile(path, { encoding: 'utf8' }, function (error, data) {
                if (error) {
                    reject(error);
                }
                else {
                    resolve(data);
                }
            });
        });
    }
    var sourceMapRegEx = /^(?:\/{2}[#@]{1,2}|\/\*)\s+sourceMappingURL\s*=\s*(data:(?:[^;]+;)+base64,)?(\S+)/;
});
//# sourceMappingURL=util.js.map